
/*******************************************************************************

 Copyright (c) 2009, Charles McGarvey
 All rights reserved.
 
 Redistribution   and   use  in  source  and  binary  forms,  with  or  without
 modification, are permitted provided that the following conditions are met:
 
   * Redistributions  of  source  code  must retain the above copyright notice,
     this list of conditions and the following disclaimer.
   * Redistributions  in binary form must reproduce the above copyright notice,
     this  list of conditions and the following disclaimer in the documentation
     and/or other materials provided with the distribution.
 
 THIS  SOFTWARE  IS  PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 AND  ANY  EXPRESS  OR  IMPLIED  WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.  IN  NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 FOR  ANY  DIRECT,  INDIRECT,  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 DAMAGES  (INCLUDING,  BUT  NOT  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES;  LOSS  OF  USE,  DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 CAUSED  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 OR  TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*******************************************************************************/

#include <sstream>
#include <cstring>

#include "settings.hh"


namespace dc {


settings::settings(int argc, char* argv[])
{
	parseArgs(argc, argv);
}


void settings::parseArgs(int argc, char* argv[])
{
	for (int i = 1; i < argc; i++)
	{
		char* where = strchr(argv[i], (int)'=');

		if (where)
		{
			std::string key(argv[i], (size_t)(where - argv[i]));
			std::string stringValue(where + 1);

			std::stringstream stream;
			stream << stringValue << std::endl;

			deserializer in(stream);

			try
			{
				serializable_ptr value(in.deserialize());
				map[key] = value;
			}
			catch (std::exception e)
			{
				map[key] = serializable_ptr(new wrapped_string(stringValue));
			}
		}
	}
}

void settings::loadFromFile(std::string filePath)
{
	deserializer in(filePath, true);

	try
	{
		serializable_ptr obj = in.deserialize();
		std::map<std::string,serializable_ptr> dict;
		if (obj && obj->get(dict))
		{
			map.insert(dict.begin(), dict.end());
		}
		else
		{
			std::cerr << "The settings file " << filePath <<
				" does not contain any valid settings." << std::endl;
		}
	}
	catch (deserializer::exception e)
	{
		std::cerr << "Cannot load settings from " << filePath <<
			" because an exception was thrown: " << e.what() << std::endl;
	}
}


} // namepsace dc

